import xml.etree.ElementTree as ET
import os
import sys
import pathlib
import typora_cos


configPath = os.path.join(*pathlib.Path.home().joinpath('.typora_cos', 'config.xml').parts) 

class Config:
    def __init__(self, secretId, secretKey, region, token, scheme, bucket):
        self.secretId = secretId
        self.secretKey = secretKey
        self.region = region
        self.token = token
        self.scheme = scheme
        self.bucket = bucket

class ConfigFactory:

    @staticmethod
    def getConfig():
        """
            返回 Config 对象
        """
        def camel(text):
            """ 转换成驼峰状 """
            return text[0].lower() + text.title()[1:].replace('_', '')

        config = dict()
        # 读取配置文件
        try:
            tree = ET.parse(configPath)
            xmlConfig = {camel(property.attrib['name']): property.attrib['value'] for property in tree.findall('property')}
            config.update(xmlConfig)
        except FileNotFoundError as err:
            print('请先执行 python -m typora_cos.qcloud --init 生成配置文件，并完成配置')
            sys.exit(1)
        if config.get('token') == 'None': config['token'] = None
        # 读取环境变量
        for name, env in {'secretId': 'QCLOUD_COS_SECRET_ID', 'secretKey': 'QCLOUD_COS_SECRET_KEY'}.items():
            value = os.environ.get(env)
            if value is not None:
                config[name] = value
        return Config(**config)

if __name__ == "__main__":
    print(ConfigFactory.getConfig().bucket)


from qcloud_cos import CosConfig
from qcloud_cos import CosS3Client

from typora_cos.factory.ConfigFactory import ConfigFactory

class ClientFactory:
    @staticmethod
    def getClient():
        # 初始化客户端 
        config  = ConfigFactory.getConfig()
        cosConfig = CosConfig(Region=config.region, SecretId=config.secretId, SecretKey=config.secretKey, Token=config.token)
        client = CosS3Client(cosConfig)
        return client


if __name__ == "__main__":
    client = ClientFactory.getClient()
    print(client.list_buckets())


# -*- coding: utf-8 -*-

import pathlib
import os
import sys
import shutil
import typora_cos

from typora_cos.service.CosClient import CosClient
from typora_cos.resource.ResourceLoader import ResourceLoader
from typora_cos.service.CosFile import CosFile

"""
    程序主入口：提供生成配置文件和上传文件功能
"""


def initConfig():
    """
        生成配置文件
    """
    configDirPath = os.path.join(*pathlib.Path.home().joinpath(".typora_cos").parts)
    configTemplatePath = os.path.join(os.path.dirname(os.path.abspath(typora_cos.__file__)), 'config.xml')
    configFilePath = os.path.join(configDirPath, 'config.xml')
    if not os.path.exists(configFilePath):
        os.makedirs(configDirPath, exist_ok=True)
        shutil.copy(configTemplatePath, configFilePath)
        print('已生成配置文件，请继续完成配置：', configFilePath)
        print('从 https://console.cloud.tencent.com/cam/capi 获取 SecretId 和 SecretKey')
        print('从 https://console.cloud.tencent.com/cos5/bucket 创建存储桶并得到存储桶名称 bucket 和 所属区域 scheme')
    else:
        print("配置文件已存在，请先删除：", configFilePath)

def upload():
    """
        从命令行读取一个或多个路径并将其文件上传到腾讯云对象存储
    """
    paths = sys.argv[1:]
    client = CosClient()
    urls = list()
    for path in paths:
        resourceLoader = ResourceLoader(path)
        cosFile = CosFile(resourceLoader.getBytes())
        url = client.upload(cosFile)
        urls.append(url)

    print("已成功上传：")
    for url in urls: print(url)

if __name__ == "__main__":
    if len(sys.argv) > 1 and sys.argv[1] == '--init':
        initConfig()
    else:
        upload()


from qcloud_cos.cos_exception import CosServiceError

from io import BytesIO

from typora_cos.factory.ClientFactory import ClientFactory
from typora_cos.factory.ConfigFactory import ConfigFactory


class CosClient:
    """
        职责：封装腾讯云SDK客户端，上传 CosFile 表示的文件到腾讯云对象存储
        设计模式：外观模式
    """
    def __init__(self):
        self.config = ConfigFactory.getConfig()
        self.client = ClientFactory.getClient()

    def upload(self, CosFile):
        url = 'https://{bucket}.cos.{region}.myqcloud.com/{key}'.format(bucket=self.config.bucket, region=self.config.region, key=CosFile.getKey())
        try:
            # 判断文件是否已经上传过
            self.client.head_object(self.config.bucket, CosFile.getKey())
            # 没有报错表示文件已经存在
            return url
        except CosServiceError as err:
            if err.get_error_code() == 'NoSuchResource':
                # 上传图片
                with BytesIO(CosFile.getBytes()) as fp:
                    self.client.put_object(
                        Bucket=self.config.bucket,
                        Body=fp,
                        Key=CosFile.getKey(),
                        StorageClass='STANDARD',
                        EnableMD5=False
                    )
                return url
            else:
                raise err

if __name__ == "__main__":
    from typora_cos.resource.ResourceLoader import ResourceLoader
    from typora_cos.service.CosFile import CosFile

    path = 'C:/Users/liaoz/Pictures/mc.jpg'
    resourceLoader = ResourceLoader(path)
    CosFile = CosFile(resourceLoader.getBytes())
    client = CosClient()
    url = client.upload(CosFile)
    print(url)


from PIL import Image

from hashlib import md5
from io import BytesIO

class CosFile:
    """
        对要Cos上的文件进行抽象（或者对要上传的文件进行抽象）
    """
    def __init__(self, bytes):
        self.bytes = bytes
        self.md5 = md5(bytes).hexdigest()
        
        with BytesIO(bytes) as fp:
            self.format = Image.open(fp).format.lower()
    
    def getKey(self):
        """
            返回上传图片的文件名
        """
        return self.md5 + '.' + self.format

    def getBytes(self):
        """
            返回上传文件的字节流
        """
        return self.bytes


import re

from typora_cos.resource.FileSystemResource import FileSystemResource
from typora_cos.resource.UrlResource import UrlResource

class ResourceLoader:
    """
        职责：从 path 加载字节流
        设计模式：策略模式 + 简单工厂方法
    """
    
    def __init__(self, path):
        if re.match(r'http|ftp', path):
            self.resource = UrlResource(path)
        else:
            self.resource = FileSystemResource(path)

    def getBytes(self):
        return self.resource.getBytes()


import setuptools

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="typora_cos",
    version="2.0.1",
    author="liaozibo",
    author_email="liaozibo@qq.com",
    description="tool for uploading typora images to qcloud cos",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitee.com/liaozb1996/typroa_qcloud_cos.git",
    packages=setuptools.find_packages(),
    package_data={
        'typora_cos': ['config.xml'],
    },
    install_requires=[
        'cos-python-sdk-v5',
        'requests',
        'Pillow',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)